from . import _init_interpolation  # noqa: F401

# autogenerated by 'robotpy-build create-imports wpimath.interpolation'
from ._interpolation import (
    TimeInterpolatableFloatBuffer,
    TimeInterpolatablePose2dBuffer,
)

__all__ = ["TimeInterpolatableFloatBuffer", "TimeInterpolatablePose2dBuffer"]
